#!/usr/bin/env python3

import subprocess
import tempfile
import os, time, sys
import re
import xml.etree.ElementTree as ET
import xml.dom.minidom as minidom
import configparser
import argparse
from termcolor import colored, cprint

from processor3D.cad import *
import gcodeParser
import processor3D.amf
import processor3D.slicer


def mtime(file):
    if os.path.exists(file):
        return os.path.getmtime(file)
    else:
        return 0


class Processor3D():
    def __init__(self, arg=None):
        super(Processor3D, self).__init__()
        self.arg = arg
        self.amf = None
        self.amf_files = {'main': None}
        self.metadata = {}
        self.default_profile = None

        parser = argparse.ArgumentParser(description='3D workflow toolbox')
        parser.add_argument('--no_amf', help='Do not save amf file', action='store_true')
        parser.add_argument('--no_stl', help='Do not save stl file', action='store_true')
        parser.add_argument('--no_png', help='Do not save pnf file', action='store_true')
        parser.add_argument('--no_gcode', help='Do not save gcode file', action='store_true')
        parser.add_argument('--no_json', help='Do not save json file', action='store_true')

        parser.add_argument('--force', help='Process all files', action='store_true')

        parser.add_argument('--slicer', help='Slicer type. Currently supported only "slic3r" (default)', default='slic3r')

        parser.add_argument('--slic3r_profile', help='Slicer config', default=None)

        parser.add_argument('--output_folder', help='Where to store output files', default=None)
        parser.add_argument('escad', help='extendSCAD (.escad) file')
        args = parser.parse_args()

        # print("ARGUMENTY", args)

        self.scad_file = args.escad
        self.basename = '.'.join(os.path.basename(self.scad_file).split('.')[:-1])

        self.source_folder = os.path.abspath(os.path.dirname(self.scad_file))
        self.dest_folder = self.source_folder
        if args.output_folder:
            self.dest_folder = args.output_folder

        self.amf_folder = os.path.abspath(os.path.join(self.dest_folder, 'amf'))
        self.json_folder = os.path.abspath(os.path.join(self.dest_folder, 'amf'))
        self.stl_folder = os.path.abspath(os.path.join(self.dest_folder, 'stl'))
        self.gcode_folder = os.path.abspath(os.path.join(self.dest_folder, 'gcode'))
        self.image_folder = os.path.abspath(os.path.join(self.dest_folder, 'img'))
        self.slicing_folder = os.path.abspath(os.path.join(self.dest_folder, 'slicing'))
        self.slic3r_config = args.slic3r_profile

        self.amf_file = os.path.join(self.amf_folder, self.basename+'.amf')
        self.stl_file = os.path.join(self.amf_folder, self.basename+'.stl')
        self.json_file = os.path.join(self.amf_folder, self.basename+'.json')
        self.png_file = os.path.join(self.amf_folder, self.basename+'.png')
        self.gcode_file = os.path.join(self.amf_folder, self.basename+'.gcode')

        if not os.path.exists(self.amf_folder):
            os.makedirs(self.amf_folder)
        if not os.path.exists(self.stl_folder):
            os.makedirs(self.stl_folder)
        if not os.path.exists(self.gcode_folder):
            os.makedirs(self.gcode_folder)
        if not os.path.exists(self.image_folder):
            os.makedirs(self.image_folder)
        if not os.path.exists(self.json_folder):
            os.makedirs(self.json_folder)
        # if not os.path.exists(self.slicing_folder): os.makedirs(self.slicing_folder)

        self.amf_file = os.path.join(self.amf_folder, self.basename+'.amf')
        self.stl_file = os.path.join(self.stl_folder, self.basename+'.stl')
        self.json_file = os.path.join(self.json_folder, self.basename+'.json')
        self.png_file = os.path.join(self.image_folder, self.basename+'.png')
        self.gcode_file = os.path.join(self.gcode_folder, self.basename+'.gcode')

        scad_mtime = mtime(self.scad_file)
        if not args.force:
            if scad_mtime < mtime(self.gcode_file) \
                and scad_mtime < mtime(self.amf_file)\
                and scad_mtime < mtime(self.png_file)\
                    and scad_mtime < mtime(self.gcode_file):
                print("Nebyla provedena zadna zmena, ukoncuji se...")
                sys.exit(0)
        else:
            print("Forcing render")

        self.load_file()
        self.process_extensions()
        self.process_render()
        self.process_slicing()

    def load_file(self, file_name=None):
        if not file_name:
            file_name = self.scad_file
        # TODO: file validation

        self.scad_file = file_name

        cprint("Loading scad file: " + file_name, 'green')
        parser = EScadParser()

        cprint("Parsing scad file", 'green')
        parser.parse_file(self.scad_file)
        self.model_extensions = parser.get_extensions()

    def process_extensions(self):
        pass

    def process_render(self):
        render = ScadRender(self.scad_file)
        self.amf = render.render_file(self.amf_file, 'amf')
        render.export_image(self.png_file, 'png')
        # render.export_stl(self.stl_file, 'stl')

        last_modificator = 'main'

        for process in self.model_extensions:
            arg = process['arguments']

            if process['command'] in ['render', 'set_modificator']:
                print("Pridavam modifikator")
                if not process['arguments'] or process['arguments'] == '':
                    pass
                else:
                    modificator_name = process['arguments'][0]
                    amf = render.render_module(os.path.join(self.amf_folder, '.amf'), modificator_name, 'amf')
                    self.amf.add_modificator(modificator_name, amf)

            if process['command'] in ['set_slicing_config']:
                if len(arg) == 1:
                    print("NASTAVUJI SLICOVACI PROFIL PRO MODEL")
                    file = self.amf.set_slicing_config(arg[0])
                    self.default_profile = file

                else:
                    self.amf.modificator(arg[1]).set_slicing_config(arg[0])

            if process['command'] in ['set_slicing_parameter']:
                if len(arg) == 2:
                    self.amf.set_slicing_parameter(arg[0], arg[1])
                else:
                    self.amf.modificator(arg[2]).set_slicing_parameter(arg[0], arg[1])

    def process_slicing(self):
        amf = self.amf.write(os.path.join(self.amf_folder, self.basename+'.amf'))
        gcode = self.amf.write(os.path.join(self.gcode_folder, self.basename+'.gcode'))
        slice = slicer.Slicer()
        slice.set_amf(amf)
        if self.slic3r_config:
            slice.set_default_profile(self.slic3r_config, overwrite=True)
        elif self.default_profile:
            print("Pouzivam profil", self.default_profile)
            slice.set_default_profile(self.default_profile, overwrite=True)

        # slice.set_amf(amf)
        self.amf.build_slicing_parameters()
        slice.slice(gcode)

        parser = gcodeParser.GcodeParser()
        parser.setMeshBBox(self.amf.get_BBox())
        self.model = parser.parseFile(gcode)
        print(self.model)


def main():
    esc = Processor3D()


if __name__ == '__main__':
    main()
